//==============================================================================
//  Verilog testbench for AdestoTech Devices AT45DQXXX parameterizable
//  Copyright 2013 Adesto Technologies Corporation, Inc.
//
//  May, 2013
//
//  Revision History:
//
//  2.1    : Post-Beta (3nd Update) changes to model
//==============================================================================

`include "AT45DQXXX.v"

`timescale 100ps / 10ps

module test;

event	DEBUG_SO;
event	DEBUG_DO;
event	DEBUG_QO;
event	BYTEBOUND;

reg	SO;
reg	SI;
reg	WPB;
reg	RSB;
reg	SCK;
reg	CSB;

reg	SO_cmp;		always @(SO) SO_cmp = SO;
reg	SI_cmp;		always @(SI) SI_cmp = SI;
reg	WPB_cmp;	always @(WPB) WPB_cmp = WPB;
reg	RSB_cmp;	always @(RSB) RSB_cmp = RSB;

wire	SO_dut;		assign SO_dut = SO;
wire	SI_dut;		assign SI_dut = SI;
wire	WPB_dut;	assign WPB_dut = WPB;
wire	RSB_dut;	assign RSB_dut = RSB;

`define AT45DQXXX_TIMING "AT45DB021E_1v65_timing.v"

AT45DQXXX dut (.CSB(CSB), .SCK(SCK), .SI(SI_dut), .WPB(WPB_dut), .RESETB(RSB_dut), .SO(SO_dut));

defparam dut.DEVICE = "AT45DB021E";
defparam dut.BINARY_OPT   = 1;	// 0: Dataflash page size; 1: Binary page size
defparam dut.QUADEN_OPT   = 1;	// 1: Initial Quad-bit read-write enabled
defparam dut.QUADEN_DIS   = 0;	// 1: Disable QEN (Program Quad-mode op-code)
defparam dut.RMW_OVERRIDE = 1;	// 1: Allow Read-Modify-Write op-codes
defparam dut.WARN_SOUT    = 1;	// 1: Enable "Incomplete byte in serial output" message
defparam dut.BADADDR_OPT  = 1;	// 1: Output XX when invalid DataFlash address supplied

defparam dut.MEMORY_FILE = "memory_ms.txt";
defparam dut.PROTECTION = "";
defparam dut.LOCKDOWN = "";
defparam dut.SECURITY = "";
defparam dut.FACTORY = "AT45DQXXX_factory.txt";

defparam dut.DEBUG_OPT0 = 1;
defparam dut.DEBUG_OPT1 = 1;
defparam dut.DEBUG_OPT2 = 0;
defparam dut.DEBUG_OPT3 = 0;
defparam dut.DEBUG_OPT4 = 0;
defparam dut.DEBUG_OPT5 = 0;
defparam dut.DEBUG_OPT6 = 0;
defparam dut.DEBUG_OPT7 = 0;

/* -------------------------------------------------------------------------- */

integer mismatch_outputs;
event MISMATCH;

initial
begin
   mismatch_outputs = 0;
end

always @(posedge SCK)
begin : CHECK_MISMATCH
   if (SI_cmp !== SI_dut)
   begin
      mismatch_outputs = mismatch_outputs + 1;
      $display ("MISMATCH: (@ %.3fms) SI cmp (%1d) != dut (%1d) in opcode %02h",
         1.0e-6*$time, SI_cmp, SI_dut, dut.opcode_reg);
      -> MISMATCH;
   end

   if (SO_cmp !== SO_dut)
   begin
      mismatch_outputs = mismatch_outputs + 1;
      $display ("MISMATCH: (@ %.3fms) SO cmp (%1d) != dut (%1d) in opcode %02h",
         1.0e-6*$time, SO_cmp, SO_dut, dut.opcode_reg);
      -> MISMATCH;
   end

   if (WPB_cmp !== WPB_dut)
   begin
      mismatch_outputs = mismatch_outputs + 1;
      $display ("MISMATCH: (@ %.3fms) WPB cmp (%1d) != dut (%1d) in opcode %02h",
         1.0e-6*$time, WPB_cmp, WPB_dut, dut.opcode_reg);
      -> MISMATCH;
   end

   if (RSB_cmp !== RSB_dut)
   begin
      mismatch_outputs = mismatch_outputs + 1;
      $display ("MISMATCH: (@ %.3fms) RSB cmp (%1d) != dut (%1d) in opcode %02h",
         1.0e-6*$time, RSB_cmp, RSB_dut, dut.opcode_reg);
      -> MISMATCH;
   end
end

/* -------------------------------------------------------------------------- */

initial
begin : TESTBENCH_
`include "at45dqxxx_stim.v"

if (test.mismatch_outputs > 0)
    $display("*** Number of output mismatches = %1d", test.mismatch_outputs);
else
    $display("*** Number of output mismatches = None");
end

initial begin
    $dumpfile("at45dqxxx_tb.vcd");
    $dumpvars(0, test);
end

endmodule

